#!/usr/bin/env python

"""
 smtp2zope.py

 Reads a email from stdin and forwards it to ZOPE
 
 Usage: smtp2zope.py URL [maxBytes]
 
 URL  = call this URL with the email as a post-request

 maxBytes = optional, but highly recommended:             
	    only forward mails < maxBytes to ZOPE

 #####
 ###  smtp2zope.py - a smtp-to-zope-gateway for ZOPE
 ##   Copyright (C) 2002   Maik Jablonski (maik.jablonski@uni-bielefeld.de)
 # 
 #    This program is distributed in the hope that it will be useful,
 #    but WITHOUT ANY WARRANTY; without even the implied warranty of
 ##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ###  GNU General Public License for more details.   
 #####
"""

import sys, StringIO, multifile, mimetools, urllib, mimify, re

# Initialise a Dictionary for storing the mail-data

zopeMail = {'headers':{},'attachments':[],'body':''}


def unpack_multifile(multifile):
   """ Given a multifile, all attachments will be mime-decoded 
       and stored in the zopeMail-Dictionary """

   global zopeMail
   
   msg      = mimetools.Message(multifile)
   maintype = msg.getmaintype()
   subtype  = msg.getsubtype()
   name     = msg.getparam('name')

   # Recurse over all nested multiparts

   if maintype == 'multipart':
       multifile.push(msg.getparam('boundary'))
       multifile.readlines()
       while not multifile.last:
           multifile.next()
           unpack_multifile(multifile)
       multifile.pop()
       return

   # Process MIME-encoded data

   plainfile = StringIO.StringIO()

   try:
       mimetools.decode(multifile,plainfile,msg.getencoding())

   # An unknown or no encoding? 7bit or 8bit or whatever... copy literal

   except ValueError:
       mimetools.copyliteral(multifile,plainfile)

   # Get the plain text as the body of the mail...
   
   if maintype=='text' and subtype=='plain' and not name:
       zopeMail['body'] = plainfile.getvalue()
       
   # Decode all the rest
  
   else:

       # No name? This is highly propably the html-encoded mail-body...

       if not name:
           name = '%s.%s' % (maintype,subtype)

       # Store the attachment in Dictionary as a list of (name,content,content-type)
		
       zopeMail['attachments'].append([name,plainfile.getvalue(),maintype+'/'+subtype])

   plainfile.close()


## Mainpart of the script

# Check if we have at least one parameter 

if len(sys.argv) == 1:
	print __doc__
	sys.exit(1)

# Get the url to call

callURL   = sys.argv[1]

# Check for optional maxBytes

if len(sys.argv) == 3:
	maxBytes  = int(sys.argv[2])
else:
	maxBytes  = 0 # means: unlimited!!!

# Get the raw mail and check its size

rawData = sys.stdin.read()

if maxBytes>0 and len(rawData)>maxBytes:
	sys.exit()

# Decode the email
	
msg     = mimetools.Message(StringIO.StringIO(rawData))  

# Extract the required email-headers & unmimify them

for (key,value) in msg.items():
	zopeMail['headers'][key] = mimify.mime_decode_header(value)
   
# Process mime encoded mail messages... works for plain mails as well...;-)

unpack_multifile(multifile.MultiFile(StringIO.StringIO(rawData)))

# Urlencode attachments as ZOPE-records

attachments=''

for (filename,filecontent,filetype) in zopeMail['attachments']:
	attachments = attachments + '&attachments.filename:records='    + urllib.quote(re.sub('[^\w.-]','',filename))
	attachments = attachments + '&attachments.filecontent:records=' + urllib.quote(filecontent)
	attachments = attachments + '&attachments.filetype:records='    + urllib.quote(filetype)

# Urlencode the headers, body & attachments for http-request

postData=urllib.urlencode(zopeMail['headers'])+'&body='+urllib.quote(zopeMail['body'])+'&rawData='+urllib.quote(rawData)

urllib.urlopen(callURL,data=postData+attachments)
